#!/bin/sh
#
# This script is executed repeatedly until the terminal is powered off.
# It can go from the configuration program to the graphical login over and over
#
# Some things must be done only once after booting. The FIRST_TIME variable 
# is used to know if this is indeed the first time through the script.

export PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/X11R6/bin
FIRST_TIME=0
TIMEOUT=0

if [ ! -f /proc/kcore ] ; then
  FIRST_TIME=1
  TIMEOUT=5
  echo "Xterminal init"

  # Get /proc and the loopback network setup

  mount -n -t proc none /proc
  /sbin/ifconfig lo 127.0.0.1
  /sbin/route add -net 127.0.0.0 netmask 255.0.0.0 dev lo

  # Starting portmap will eliminate some error messages with 2.2 kernels.
  case `uname -r` in
    2.2.*)
      /sbin/portmap
    ;;
  esac
fi

# The terminal should have a directory named after its ethernet address
# on the server.

BASEDIR=/var/xterminals
NAME=`/bin/getrefdir`
CTRLDIR=$BASEDIR/$NAME
SERVER=`/sbin/getdefserver defserver`

# Mount the configuration directory if it exists

if [ ! -d /etc/X11/sysconfig ]; then
  echo "Checking xterminal setup directory $CTRLDIR"
  mount -n $SERVER:$CTRLDIR /etc/X11
fi

# No configuration directory for this terminal, create one

if [ ! -d /etc/X11/sysconfig ]; then
  echo
  echo "********************************************************"
  echo Missing xterminal setup directory $CTRLDIR
  echo Trying to create one with default values

  # Take as "default" value the directory named "preferred", or the most
  # recent configuration directory, of the one named "default" (supplied
  # with the kit) if there are none others.

  if mount -n $SERVER:$BASEDIR /mnt/xterminals; then
    if [ -d /mnt/xterminals/preferred/. ]; then 
      DEFAULT=preferred/.
    else
      DEFAULT=`ls -t /mnt/xterminals | head -1`
    fi

    if [ -n "$DEFAULT" -a "$DEFAULT" != "$NAME" ]; then
		cp -a /mnt/xterminals/$DEFAULT /mnt/xterminals/$NAME
	else
		mkdir -p /mnt/xterminals/$NAME \
			&& mkdir -p /mnt/xterminals/$NAME/sysconfig \
			&& mkdir -p /mnt/xterminals/$NAME/tmp \
			&& /etc/initdev /mnt/xterminals/$NAME/dev
			echo APPSERVER=defserver >/mnt/xterminals/$NAME/sysconfig/xterminal
			echo FONTSERVER=defserver >>/mnt/xterminals/$NAME/sysconfig/xterminal
    fi

    # Remember that this is a new copied configuration. Do not keep
    # the plug'n play configuration as this is not transferable.
    # The configuration directory is touched to change the date
    # making this the newest configuration directory.

    touch /mnt/xterminals/$NAME
    touch /mnt/xterminals/$NAME/sysconfig/NEW
    rm -f /mnt/xterminals/$NAME/isapnp.conf

    # The terminal name is changed to obtain unique names

    XTERMNAME=`cat /mnt/xterminals/$NAME/sysconfig/TERMNAME | \
        sed -e 's/^\(.*[^0-9]\)\([0-9][0-9]*\)[ ]*$/\1/'`
    XTERMNUMBER=`cat /mnt/xterminals/$NAME/sysconfig/TERMNAME | \
        sed -e 's/^\(.*[^0-9]\)\([0-9][0-9]*\)[ ]*$/\2/'`
    if [ -z "$XTERMNUMBER" ]; then XTERMNUMBER=0; fi
    XTERMNUMBER=`expr $XTERMNUMBER + 1`
    echo "$XTERMNAME$XTERMNUMBER" >/mnt/xterminals/$NAME/sysconfig/TERMNAME

    # Finally mount the configuration directory

    umount -n /mnt/xterminals
    mount -n $SERVER:$CTRLDIR /etc/X11
  fi
fi

# At this point we should have the configuration directory mounted as /etc/X11
# We can mount the /dev directory too if not already mounted.

if [ ! -d /etc/X11/sysconfig ]; then
  echo Make sure directory /var/xterminals is exported read/write
elif [ ! -e /dev/mouse ]; then
  echo "Mounting xterminal setup directory"
  mount -n $SERVER:$CTRLDIR/dev /dev
fi

# Start the different services only if this is the first time after booting.
# If the configuration program changes some of these parameters, it is its
# responsability to restart the services as needed.

if [ ${FIRST_TIME} -ne 0 ]; then

  # If this is a new copied config, do not enable swap yet.
  if [ ! -f /etc/sysconfig/NEW ]; then /sbin/enable-swap; fi

  /etc/rc.d/init.d/keytable start
  if [ -f /etc/X11/isapnp.conf ]; then
    echo Initialising PnP devices
    /sbin/isapnp /etc/X11/isapnp.conf
  fi
  if [ -f /etc/X11/conf.modules ] ; then
    echo Loading sound modules
    modprobe sound 
    modprobe midi
    modprobe sound-slot-0
  fi
  if [ -x /usr/sbin/remaudioserv ] ; then
    echo Starting the audio server
    /usr/sbin/remaudioserv --silentmount &
  fi
  if [ -x /usr/sbin/litefs-xterminal ] ; then
    echo Starting the cdrom and floppy file server
    /usr/sbin/litefs-xterminal
  fi
  if [ -f /etc/sysconfig/printserv ] ; then
    /sbin/printserv
  fi
fi

# Ask what to do: login/configure/shell

if [ -d /etc/X11/sysconfig ]; then
  if [ ! -f /etc/sysconfig/NEW ]; then
    # We are configured, all choices are available
    if [ -f /configuring ]; then
      # Configuration is allowed
      if [ -s /configuring ]; then
        xtermmenu 0 $TIMEOUT
      else
        xtermmenu 0 0
      fi
      CHOICE=$?
    else
      CHOICE=0
    fi
  else
    # This was never really configured, login is not available
    xtermmenu 1 $TIMEOUT
    CHOICE=$?
  fi
else
  # Something is wrong, the shell is the only alternative, no menu
  CHOICE=2
fi

case $CHOICE in
  # We just want a shell
  2)
    reset; clear
    exec /bin/sh
  ;;

  # Go in configuration mode, make the other configurations in BASEDIR
  # available as templates
  1) 
    echo Trying to mount $BASEDIR on $SERVER
    if mount -n $SERVER:$BASEDIR /mnt/xterminals; then
      xtermconfig ${CTRLDIR}
      reset; clear
      umount -n /mnt/xterminals
    else
      echo Make sure directory $BASEDIR is exported read/write
      echo via NFS
    fi
    exit 0
  ;;

  # The real thing, we are going in graphical login mode
  0)
    if [ -f /etc/sysconfig/xterminal ] ; then
      . /etc/sysconfig/xterminal
    fi
    if [ "$APPSERVER" = "" ] ; then
      APPSERVER=defserver
    fi
    # Resolve defserver to its effective address
    APPSERVER=`/sbin/getdefserver $APPSERVER`
    FONTSERVER=`/sbin/getdefserver $FONTSERVER`
    echo Starting X on application server $APPSERVER
    if [ "$FONTSERVER" != "" ] ; then
      FPSPEC="-fp tcp/$FONTSERVER:7100"
    fi
    if [ "$XKBDISABLE" = "true" ] ; then
      KBSPEC=-kb
    else
      # make sure there is a "compiled" directory
      # we need a ramdisk for that
      COMPILED=/usr/X11R6/lib/X11/xkb/compiled
      if [ ! -f $COMPILED/dummy ] ; then
	/sbin/mkfs.minix /dev/ram 150
	mount -n -t minix /dev/ram $COMPILED
	# Just a flag to tell xgo that the ramdisk is on
	>$COMPILED/dummy
      fi
    fi
    echo /etc/X11/X -nolock -nolisten local $KBSPEC $FPSPEC -query $APPSERVER
    exec /etc/X11/X -nolock -nolisten local $KBSPEC $FPSPEC -query $APPSERVER
  ;;
esac
